"""Just constants useful for testing across test types."""

# Following constants used by upload tests.
ONE_TO_SIX_WITH_SPACES = "1 2 3\n4 5 6\n"
ONE_TO_SIX_WITH_TABS = "1\t2\t3\n4\t5\t6\n"
ONE_TO_SIX_ON_WINDOWS = "1\t2\t3\r4\t5\t6\r"
ONE_TO_SIX_WITH_SPACES_ON_WINDOWS = "1 2 3\r4 5 6\r"
ONE_TO_SIX_WITH_TABS_NO_TRAILING_NEWLINE = "1\t2\t3\n4\t5\t6"
